#include "area.h"
#include <stdlib.h>
#include <string.h>


struct AREA *createArea(struct AREA *area, const char *name) {
	area = calloc(1, sizeof(struct AREA));
	area->points = NULL;
	if (name != NULL) {
		size_t len = strlen(name);
		area->name = calloc(len+2, sizeof(char));
		strcpy(area->name, name);
	}
	else area->name = NULL;
	return area;
}

char updateArea(struct AREA *area, const char *name,
		const struct TIME_INTERVAL *time_interval,
		const struct POINT *points, const int points_count,
		const float mu) {
	if (name != NULL) {
		if (area->name != NULL)
			free(area->name);
		size_t len = strlen(name);
		area->name = calloc(len + 2, sizeof(char));
		strcpy(area->name, name);
	}
	if (time_interval != NULL) {
		area->time_interval.begin_t = time_interval->begin_t;
		area->time_interval.end_t = time_interval->end_t;
	}
	if (points != NULL) {
		if (area->points != NULL)
			destroyPoints(area->points);
		area->points = calloc(points_count, sizeof(struct POINT));
	}
	area->mu = mu;
	return 0;
}


void deleteArea(struct AREA *area) {
	if (area == NULL) return;
	/* Сначало удалить все подключенные структуры */
	if (area->name != NULL)
		free(area->name);
	if (area->points != NULL)
		destroyPoints(area->points);
	free(area);
}

/* равенство частей */
int isEqualArea(const struct AREA *first, const struct AREA *second,
		const char flag_name, const struct POINT *delta_coord,
		const time_t delta_time) {
	/* Сравнивать имена или нет */
	if (flag_name == 1)
		if (strcmp(first->name, second->name))
			return 0;
	/* Сравнить время */
	if (inTimeInterval(&first->time_interval, &second->time_interval,
			delta_time) == 0)
		return 0;
	/* Сравниваем координаты */
	if (first->points_count != second->points_count)
		return 0;
	int pcount;
	for (pcount = 0; pcount < first->points_count; pcount++)
		if (isEqualPoint(&first->points[pcount],
				&second->points[pcount],
				delta_coord) == 0)
			return 0;
	return 1;
}


/*
(x - x0)^2 + (y - y0)^2 + (z - z0)^2 <= R^2
x0, y0, z0 - координаты центра(точки)

more or equal
less or equal
*/

int isAreaIncludesPoint(const struct AREA *area,
		const struct POINT *point) {
	int count;
	for (count = 0; count < area->points_count; count++) {
		if (area->points[count].x == point->x &&
		area->points[count].y == point->y &&
		area->points[count].z == point->z &&
		area->points[count].radius == point->radius &&
		area->points[count].add == point->add)
			return 1;
	}
	return 0;
}

int isAreaIncludesTimeInterval(const struct AREA *area,
		const struct TIME_INTERVAL *time_interval,
		const time_t delta_time) {
	return inTimeInterval(&area->time_interval, time_interval,
			delta_time);
}

/* объединение частей */
int addArea(struct AREA *result, const struct AREA *added_area) {
	if (result == NULL) return 1;
	if (added_area == NULL) return 2;
	if (result->time_interval.begin_t >
			added_area->time_interval.end_t ||
		added_area->time_interval.begin_t >
			result->time_interval.end_t)
				return 3;
	int count;
	for (count = 0; count < added_area->points_count; count++) {
		result->points = addPoint(result->points,
				result->points_count,
				&added_area->points[count]);
		result->points_count++;
	}
	return 0;
}

/* выделение частей */
int extractArea(struct AREA *result, const struct AREA *extracted_area) {
/* Выделение части.
Пока отложить.
ЗДЕСЬ АЛГОРИТМ ФОРЕЛЬ.
Под ИНС задача нахождения центра масс для геометрически разных объектов.
Выход ряд сфер перекрывающих всю территорию.
Решить задачу алгоритмически. Попробовать вывести формулу. */
	return 0;
}

/* Вспомогательные функции */
void fillArea(struct AREA *area, const char *name,
		const struct TIME_INTERVAL *time_interval,
		const struct POINT *points, const int points_count,
		const float mu) {
	/* Очищаем предыдущее значение */
	if (area->name != NULL) {
		free(area->name);
		area->name = NULL;
	}
	if (area->points != NULL) {
		destroyPoints(area->points);
		area->points = NULL;
	}
	/* Задаем новое значение */
	area->name = calloc(1, strlen(name)+2);
	strcpy(area->name, name);
	area->time_interval.begin_t = time_interval->begin_t;
	area->time_interval.end_t = time_interval->end_t;
	area->points = calloc(points_count, sizeof(struct POINT));
	memcpy(area->points, points, points_count *
			sizeof(struct POINT));
	area->points_count = points_count;
	area->mu = mu;
}

void copyArea(struct AREA *area1, const struct AREA *area2) {
	/* Очищаем предыдущее значение */
	if (area1->name != NULL) {
		free(area1->name);
		area1->name = NULL;
	}
	if (area1->points != NULL) {
		destroyPoints(area1->points);
		area1->points = NULL;
	}
	/* Задаем новое значение */
	area1->name = calloc(1, strlen(area2->name)+2);
	strcpy(area1->name, area2->name);
	area1->time_interval.begin_t = area2->time_interval.begin_t;
	area1->time_interval.end_t = area2->time_interval.end_t;
	area1->points = calloc(area2->points_count,
			sizeof(struct POINT));
	memcpy(area1->points, area2->points,
			area2->points_count * sizeof(struct POINT));
	area1->points_count = area2->points_count;
	area1->mu = area2->mu;
}

