#ifndef AREA_H
#define AREA_H

#include "../point/point.h"
#include "../time_interval/time_interval.h"

/* Область */
#pragma pack(push, 1)
struct AREA {
	char *name;
	/* Разное время, разные территории */
	struct TIME_INTERVAL time_interval;
	/* Точки из которых состоит область */
	struct POINT *points;
	int points_count;
	float mu;
};
#pragma pack(pop)

/* объединение частей, выделение частей, равенство частей */
int isEqualArea(const struct AREA *first, const struct AREA *second,
		const char flag_name, const struct POINT *delta_coord,
		const time_t delta_time);
/* 0 - добавление без ошибок, 
 * 1 - отсутствие указателя на результат,
 * 2 - отсутствие указателя на добавляемую структуру,
 * 3 - территории с расхождением по времени */
int addArea(struct AREA *result, const struct AREA *added_area);
int extractArea(struct AREA *result, const struct AREA *extracted_area);

/* Проверка вхождения точек и интервалов времени */
int isAreaIncludesPoint(const struct AREA *area, const struct POINT *point);
int isAreaIncludesTimeInterval(const struct AREA *area, const struct
		TIME_INTERVAL *time_interval, const time_t delta_time);

/* Создание, обновление, удаление территорий */
struct AREA *createArea(struct AREA *area, const char *name);
char updateArea(struct AREA *area, const char *name,
		const struct TIME_INTERVAL *time_interval,
		const struct POINT *points, const int points_count,
		const float mu);
void deleteArea(struct AREA *area);

/* Вспомогательные функции */
void fillArea(struct AREA *area, const char *name,
		const struct TIME_INTERVAL *time_interval,
		const struct POINT *points, const int points_count,
		const float mu);
void copyArea(struct AREA *area1, const struct AREA *area2);

#endif

