#!/usr/sbin/python3

import ctypes as ct
import struct as stt

class PyPoint:
    def __init__(self, x, y, z, radius, add):
        self.x = x
        self.y = y
        self.z = z
        self.radius = radius
        self.add = add

class PyTimeInterval:
    def __init__(self, begin_t, end_t):
        self.begin_t = begin_t
        self.end_t = end_t

class PyPath:
    def __init__(self, name, time_interval, points, points_count, mu):
        self.name = name
        self.time_interval = time_interval
        self.points = points
        self.points_count = points_count
        self.mu = mu

class PyArea:
    def __init__(self, name, time_interval, points, points_count, mu):
        self.name = name
        self.time_interval = time_interval
        self.points = points
        self.points_count = points_count
        self.mu = mu

class PyPersonalUrbanosfere:
    def __init__(self, name, areas, areas_count, paths, paths_count):
        self.name = name
        self.area = areas
        self.areas_count = areas_count
        self.paths = paths
        self.paths_count = paths_count

class CPoint(ct.Structure):
    _fields_ = [
        ('x', ct.c_float),
        ('y', ct.c_float),
        ('z', ct.c_float),
        ('radius', ct.c_int),
        ('add', ct.c_char)
    ]

class CTimeInterval(ct.Structure):
    _fields_ = [
        ('begin_t', ct.c_long),
        ('end_t', ct.c_long)
    ]

class CPath(ct.Structure):
    _fields_ = [
        ('name', ct.POINTER(ct.c_char)),
        ('time_interval', ct.POINTER(CTimeInterval)),
        ('points', ct.POINTER(CPoint)),
        ('pints_count', ct.c_int),
        ('mu', ct.c_float)
    ]

class CArea(ct.Structure):
    _fields_ = [
        ('name', ct.POINTER(ct.c_char)),
        ('time_interval', ct.POINTER(CTimeInterval)),
        ('points', ct.POINTER(CPoint)),
        ('points_count', ct.c_int),
        ('mu', ct.c_float)
    ]

class CPersonalUrbanosfere(ct.Structure):
    _fields_ = [
            ('name', ct.c_char),
            ('areas', ct.POINTER(CArea)),
            ('areas_count', ct.c_int),
            ('paths', ct.POINTER(CPath)),
            ('paths_count', ct.c_int)
    ]

if __name__ == '__main__':
    libc = ct.CDLL("./libPU.so.0.0")

#    libc.equalPU.argtypes = [ct.POINTER(CPersonalUrbanosfere),
#                             ct.POINTER(CPersonalUrbanosfere),
#                             ct.c_char, ct.POINTER(CPoint), ct.c_long]
#    libc.equalPU.restype = ct.c_char
#    first_pu = CPersonalUrbanosfere()
#    second_pu = CPersonalUrbanosfere()
#    flag_name = 1
#    delta_coord = CPoint()
#    delta_time = 1
#    libc.equalPU(first_pu, second_pu, flag_name, delta_coord, delta_time)

    libc.addArea.argtypes = [ct.POINTER(CArea), ct.POINTER(CArea)]
    libc.addArea.restype = ct.c_int
    result = CArea()
    added_area = CArea()
    libc.addArea(result, added_area)


    # inTimeInterval
    # char inTimeInterval(const struct TIME_INTERVAL *first,
    # const struct TIME_INTERVAL *second,
    # const time_t delta_time);
    libc.inTimeInterval.argtypes = [
            ct.POINTER(CTimeInterval),
            ct.POINTER(CTimeInterval),
            ct.c_long
    ]
    libc.inTimeInterval.restype = ct.c_char
    first_tt = CTimeInterval()
    first_tt.begin_t = 10
    first_tt.end_t = 20
    second_tt = CTimeInterval()
    second_tt.begin_t = 15
    second_tt.end_t = 25
    print("first = [", first_tt.begin_t, ",",
            first_tt.end_t, "], second = [",
            second_tt.begin_t, ",", second_tt.end_t, "], answer =",
            libc.inTimeInterval(first_tt, second_tt, 10))

    # timeIll
    # char timeIll(float p, float f, float alpha);
    libc.timeIll.argtypes = [ct.c_float, ct.c_float, ct.c_float]
    libc.timeIll.restype = ct.c_char
    print("p = 0.75, f = 57, alfa = 0,5, answer =",
            libc.timeIll(0.75, 0.57, 0.5))
    print("p = 0.25, f = 57, alfa = 0,5, answer =",
            libc.timeIll(0.25, 0.57, 0.5))

    # pfs
    # float pfs(float p, float f);
    libc.pfs.argtypes = [ct.c_float, ct.c_float]
    libc.pfs.restype = ct.c_float
    print("p = 0.34, f = 0.65, pfs =", libc.pfs(0.34, 0.65))


